package com.wtkj.entity;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.springblade.core.mp.base.BaseEntity;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.NotNull;
import java.util.Date;

/**
 * @author Blizzard
 * @create at 2023-09-15 14:35
 * @describe
 */
@EqualsAndHashCode(callSuper = true)
@Data
@TableName("blade_task")
public class Task extends BaseEntity {
	private static final long serialVersionUID = 1L;


	@JsonSerialize(using = ToStringSerializer.class)
	@TableField("project_id")
	@NotNull(message = "projectId can't be null")
	private Long projectId;

	@TableField("title")
	private String title;

	@ApiModelProperty(value = "派遣给哪个机构:顶级机构")
	@NotNull(message = "orgDeptId不可为空")
	@TableField("org_dept_id")
	private Long orgDeptId;

	@TableField("task_status")
	@ApiModelProperty("任务状态：1.待确认 2.进行中 3.已提交 4.已完成 5.已取消")
	private Integer taskStatus = 1;

	@TableField("level")
	@ApiModelProperty("优先级")
	private Integer level;

	@TableField("category")
	@ApiModelProperty("类别：可研，方案、其他等")
	private String category;

	@TableField("tags")
	@ApiModelProperty("标签：新写、修改等")
	private String tags;

	@TableField("start_time")
	@ApiModelProperty("开始时间")
	@DateTimeFormat(pattern = "yyyy-MM-dd")
	@JsonFormat(pattern = "yyyy-MM-dd")
	private Date startTime;

	@TableField("end_time")
	@ApiModelProperty("截止时间")
	@DateTimeFormat(pattern = "yyyy-MM-dd")
	@JsonFormat(pattern = "yyyy-MM-dd")
	private Date endTime;

	@TableField("is_confirmed")
	@ApiModelProperty("是否已经确认完成")
	private Integer isConfirmed;

	@TableField("confirm_time")
	@ApiModelProperty("任务完成确认时间")
	@DateTimeFormat(pattern = "yyyy-MM-dd")
	@JsonFormat(pattern = "yyyy-MM-dd")
	private Date confirmTime;

	@JsonSerialize(using = ToStringSerializer.class)
	@ApiModelProperty(value = "确认任务完成的用户")
	@TableField("confirm_user")
	private Long confirmUser;

	@TableField("remark")
	@ApiModelProperty("备注")
	private String remark;

	@TableField("task_process")
	@ApiModelProperty("任务进展")
	private String taskProcess;

	@TableField("related_ids")
	@ApiModelProperty("参考文件的主键id")
	private String relatedIds;

	@ApiModelProperty(value = "项目经理")
	@TableField("project_manager")
	private Long projectManager;

	@ApiModelProperty(value = "审查人")
	@TableField("check_user")
	private String checkUser;

	@ApiModelProperty(value = "实际执行部门")
	@TableField("execute_dept")
	private Long executeDept;

	@ApiModelProperty(value = "执行人")
	@TableField("execute_user")
	private String executeUser;

	@TableField("year")
	private Integer year;

	@TableField("month")
	private Integer month;

}
