package com.wtkj.service.impl;

import com.wtkj.entity.DeptToDept;
import com.wtkj.mapper.DeptToDeptMapper;
import com.wtkj.service.IDeptToDeptService;
import com.wtkj.vo.AuthUserInfo;
import lombok.AllArgsConstructor;
import org.springblade.core.mp.base.BaseServiceImpl;
import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.BeanUtil;
import org.springblade.core.tool.utils.Func;
import org.springblade.core.tool.utils.StringUtil;
import org.springblade.system.entity.Dept;
import org.springblade.system.feign.ISysClient;
import org.springblade.system.user.entity.User;
import org.springblade.system.user.feign.IUserClient;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

import static com.wtkj.config.MagicValue.ZERO;

/**
 * @author Blizzard
 * @create at 2023-09-14 11:11
 * @describe
 */
@Service
@AllArgsConstructor
public class DeptToDeptServiceImpl extends BaseServiceImpl<DeptToDeptMapper, DeptToDept> implements IDeptToDeptService {

	private final ISysClient sysClient;
	private final IUserClient userClient;

	@Override
	public AuthUserInfo userInfo() {
		BladeUser user = AuthUtil.getUser();
		if (user != null) {
			AuthUserInfo userInfo = Objects.requireNonNull(BeanUtil.copy(user, AuthUserInfo.class));
			String deptIdStr = userInfo.getDeptId();
			if (StringUtil.isNotBlank(deptIdStr)) {
				R<Dept> rpc = sysClient.getDept(Long.valueOf(deptIdStr));
				if (rpc.isSuccess()) {
					userInfo.setDeptCategory(rpc.getData().getDeptCategory());
					userInfo.setDeptName(rpc.getData().getDeptName());
					String ancestors = rpc.getData().getAncestors();
					// ancestors 0,1689540492698267649
					if (ancestors != null && ancestors.contains(String.valueOf(ZERO))) {
						List<Long> longList = Func.toLongList(ancestors);
						if (!CollectionUtils.isEmpty(longList)) {
							if (longList.size() > 2) {
								userInfo.setTopDept(longList.get(1));
							} else {
								userInfo.setTopDept(Long.valueOf(deptIdStr));
							}
						}
					}
				}
			}
			Long userId = user.getUserId();
			R<User> userR = userClient.userInfoById(userId);
			if (userR.isSuccess()) {
				userInfo.setUser(userR.getData());
			}
			return userInfo;
		}
		return null;
	}

	@Override
	public List<Dept> getDept(Long topDept, String areaCode, String name, Integer category) {
		List<Dept> result = new ArrayList<>();
		R<List<Dept>> rpc = sysClient.getDeptByAreaCodeAndCategory(areaCode, name, category);
		if (rpc.isSuccess()) {
			List<Dept> data = rpc.getData();
			if (!CollectionUtils.isEmpty(data)) {
				result.addAll(data);
			}
		}
		//机构关联的机构
		if (topDept != null) {
			List<DeptToDept> list = this.getByTopDept(topDept);
			if (!CollectionUtils.isEmpty(list)) {
				List<Dept> deptList = Objects.requireNonNull(BeanUtil.copy(list, Dept.class));
				result.addAll(deptList);
			}
		}
		return result;
	}

	@Override
	public List<DeptToDept> getByTopDept(Long topDept) {
		return baseMapper.selectByTopDept(topDept);
	}


}
