package com.wtkj.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.wtkj.dto.ProjectTaskPageDTO;
import com.wtkj.entity.FileAndFolder;
import com.wtkj.entity.Task;
import com.wtkj.entity.TaskFile;
import com.wtkj.mapper.TaskMapper;
import com.wtkj.service.IAsyncService;
import com.wtkj.service.IFileAndFolderService;
import com.wtkj.service.ITaskFileService;
import com.wtkj.service.ITaskService;
import com.wtkj.vo.MyTaskStatistics;
import com.wutong.file.feign.IFileClient;
import com.wutong.file.vo.FileVO;
import lombok.AllArgsConstructor;
import org.springblade.core.mp.base.BaseServiceImpl;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.DateUtil;
import org.springblade.core.tool.utils.Func;
import org.springblade.core.tool.utils.StringUtil;
import org.springblade.system.user.entity.User;
import org.springblade.system.user.feign.IUserClient;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.time.LocalDate;
import java.util.*;

/**
 * @author Blizzard
 * @create at 2023-09-15 15:18
 * @describe
 */
@AllArgsConstructor
@Service
public class TaskServiceImpl extends BaseServiceImpl<TaskMapper, Task> implements ITaskService {

	private final IAsyncService asyncService;
	private final ITaskFileService taskFileService;
	private final IFileAndFolderService fileAndFolderService;
	private final IFileClient fileClient;
	private final IUserClient userClient;

	@Override
	public boolean submit(Task task) {
		boolean flag = false;
		Long taskId = task.getId();
		if (taskId == null) {
			LocalDate now = LocalDate.now();
			Integer year = now.getYear();
			Integer month = now.getMonthValue();
			task.setYear(year);
			task.setMonth(month);
			flag = this.save(task);
			asyncService.createTaskLog(task);
		} else {
			//执行的顶级单位
			Long userId = AuthUtil.getUserId();
			Task byId = this.getById(taskId);
			if (byId != null) {
				//1.指定了执行单位
				if ((byId.getOrgDeptId() == null || byId.getOrgDeptId() < 0L) && task.getOrgDeptId() != null && task.getOrgDeptId() > 0L) {
					asyncService.updateTask(1, userId, task);
				}
				//2.指定任务执行部门
				if ((byId.getExecuteDept() == null || byId.getExecuteDept() < 0L) && task.getExecuteDept() != null && task.getExecuteDept() > 0L) {
					asyncService.updateTask(2, userId, task);
				}
				//3.指定项目经理
				if ((byId.getProjectManager() == null || byId.getProjectManager() < 0L) && task.getProjectManager() != null && task.getProjectManager() > 0L) {
					asyncService.updateTask(3, userId, task);
				}
				//4.指定任务执行者
				if (StringUtil.isBlank(byId.getExecuteUser()) && StringUtil.isNotBlank(task.getExecuteUser())) {
					asyncService.updateTask(4, userId, task);
				}
				//5.指定任务审查者
				if (StringUtil.isBlank(byId.getCheckUser()) && StringUtil.isNotBlank(task.getCheckUser())) {
					asyncService.updateTask(5, userId, task);
				}
			}
			flag = this.updateById(task);
		}
		return flag;
	}

	@Override
	public IPage<Task> projectTaskPage(ProjectTaskPageDTO dto, IPage<Task> page) {
		LambdaQueryWrapper<Task> lqw = new LambdaQueryWrapper<>();
		lqw.eq(Task::getProjectId, dto.getProjectId());
		if (dto.getDeptCategory().equals(3)) {
			//是业主
			dto.setOrgDeptId(null);
		}
		if (dto.getName() != null) {
			lqw.like(Task::getTitle, dto.getName());
		}
		if (dto.getOrgDeptId() != null) {
			lqw.eq(Task::getOrgDeptId, dto.getOrgDeptId());
		}
		if (dto.getTaskStatus() != null) {
			lqw.eq(Task::getTaskStatus, dto.getTaskStatus());
		}
		lqw.orderByAsc(Task::getTaskStatus);
		return baseMapper.selectPage(page, lqw);
	}

	@Override
	public boolean uploadFile(Long taskId, String ids) {
		boolean flag = false;
		if (ids != null) {
			List<TaskFile> files = new ArrayList<>();
			List<Long> fileIds = Func.toLongList(ids);
			fileIds.forEach(id -> {
				//判断这个文件是否已经存在
				TaskFile file = taskFileService.getByBladeFileId(taskId, id);
				//不存在 才保存
				if (file == null) {
					TaskFile taskFile = new TaskFile();
					taskFile.setTaskId(taskId);
					taskFile.setBladeFileId(id);
					files.add(taskFile);
				}
			});
			//提交任务消息
			asyncService.submitTaskMessage(AuthUtil.getUserId(), this.getById(taskId));
			flag = taskFileService.saveBatch(files);
		}
		return flag;
	}

	@Override
	public boolean moveFile(Long stageId, Long parentId, String fileIds) {

		boolean flag = false;
		List<Long> ids = Func.toLongList(fileIds);
		if (!CollectionUtils.isEmpty(ids)) {
			List<FileAndFolder> files = new ArrayList<>();
			FileAndFolder folder = fileAndFolderService.getById(parentId);
			if (folder != null) {
				ids.forEach(id -> {
					R<FileVO> rpc = fileClient.findById(id);
					if (rpc.isSuccess()) {
						FileAndFolder file = new FileAndFolder();
						file.setProjectId(folder.getProjectId());
						file.setStageId(stageId);
						file.setParentId(parentId);
						file.setType(1);
						file.setTitle(rpc.getData().getOriginalFileName());
						file.setBladeFileId(id);
						files.add(file);
					}
				});
				flag = fileAndFolderService.saveBatch(files);
			}
		}
		return flag;
	}

	@Override
	public boolean deleteFile(String ids) {
		return taskFileService.removeBatchByIds(Func.toLongList(ids));
	}

	@Override
	public IPage<Task> myTaskPage(Integer type, Integer isGroup, Integer year, Integer month, IPage<Task> page) {

		Long userId = AuthUtil.getUserId();
		LambdaQueryWrapper<Task> lqw = new LambdaQueryWrapper<>();
		//当前年 当前月
		LocalDate currentDate = LocalDate.now();
		int currentYear = currentDate.getYear();
		int currentMonth = currentDate.getMonthValue();
		lqw.eq(Task::getYear, year == null ? currentYear : year);
		lqw.eq(Task::getMonth, month == null ? currentMonth : month);

		//1.全部 2.我创建的 3.我执行的
		if (type == 1) {
			lqw.and(wq -> {
				wq.eq(Task::getCreateUser, userId);
				wq.or().like(Task::getExecuteUser, String.valueOf(userId));
			});
		} else if (type == 2) {
			lqw.eq(Task::getCreateUser, userId);
		} else {
			lqw.like(Task::getExecuteUser, String.valueOf(userId));
		}
		if (isGroup == 1) {
			lqw.groupBy(Task::getProjectId, Task::getCreateTime);
			lqw.orderByDesc(Task::getProjectId, Task::getCreateTime);
		} else {
			lqw.orderByDesc(Task::getCreateTime);
		}
		return baseMapper.selectPage(page, lqw);
	}

	@Override
	public MyTaskStatistics taskStatistics(Long projectId, Integer year, Integer month) {
		MyTaskStatistics statistics = new MyTaskStatistics();
		Long userId = AuthUtil.getUserId();
		List<Task> tasks = null;
		if (projectId == null) {
			//个人任务统计
			tasks = this.listByUser(userId, year, month);
		} else {
			//项目下的所有任务
			tasks = this.listByProjectId(projectId);
		}
		if (!CollectionUtils.isEmpty(tasks)) {
			//全部任务数
			statistics.setAll(tasks.size());
			tasks.forEach(task -> {
				//任务确认状态
				Integer isConfirmed = task.getIsConfirmed();
				//任务状态 1.待确认 2.进行中 3.已提交 4.已完成 5.已取消
				Integer taskStatus = task.getTaskStatus();
				if (isConfirmed.equals(1)) {
					//已完成
					statistics.setCompleted(statistics.getCompleted() + 1);
				}
				Date now = DateUtil.now();
				Date endTime = task.getEndTime();
				if (endTime != null) {
					//过了截至时间且未确认的任务为已逾期
					if (endTime.before(now) && isConfirmed.equals(0)) {
						statistics.setOverdue(statistics.getOverdue() + 1);
					}

					//待确认 已经提交但未确认且未逾期的任务为待确认
					if (endTime.after(now) && taskStatus.equals(3) && isConfirmed.equals(0)) {
						statistics.setToConfirmed(statistics.getToConfirmed() + 1);
					}

					//今日到期
					String endStr = DateUtil.format(endTime, "yyyy-MM-dd");
					String nowStr = DateUtil.format(now, "yyyy-MM-dd");
					if (endStr.equals(nowStr)) {
						statistics.setTodayToExpire(statistics.getTodayToExpire() + 1);
					}

					//逾期完成
					if (endTime.before(now) && isConfirmed.equals(1)) {
						statistics.setOverdueComplete(statistics.getOverdueComplete() + 1);
					}
				}
			});
			//未完成
			statistics.setNotCompleted(statistics.getAll() - statistics.getCompleted());
		}
		return statistics;
	}


	@Override
	public List<Task> listByUser(Long userId, Integer year, Integer month) {
		LambdaQueryWrapper<Task> lqw = new LambdaQueryWrapper<>();
		if (year != null) {
			lqw.eq(Task::getYear, year);
		}
		if (month != null) {
			lqw.eq(Task::getMonth, month);
		}
		lqw.like(Task::getExecuteUser, String.valueOf(userId));
		lqw.orderByAsc(Task::getEndTime);
		return this.list(lqw);
	}

	//todo 删除任务是否需要事务删除任务相关？
	@Override
	public boolean deleteTask(List<Long> taskIds) {
		return false;
	}

	@Override
	public List<Task> listByProjectId(Long projectId) {
		LambdaQueryWrapper<Task> lqw = new LambdaQueryWrapper<>();
		lqw.eq(Task::getProjectId, projectId);
		lqw.orderByAsc(Task::getEndTime);
		return this.list(lqw);
	}

	@Override
	public Set<Long> getByExecute(Long deptId, String userId) {
		return baseMapper.selectByExecute(deptId, userId);
	}

	@Override
	public Set<Long> getByUser(String userId) {
		return baseMapper.selectByUser(userId);
	}

	@Override
	public Set<Long> todayTask(Long userId) {
		String now = DateUtil.format(new Date(), "yyyy-MM-dd");
		return baseMapper.selectTodayTask(userId, now);
	}

	@Override
	public List<Task> todayTodo(Long userId, String date) {
		return baseMapper.todayTo(String.valueOf(userId), date);
	}

	@Override
	public List<Task> todayExpire(Long userId, String date) {
		return baseMapper.todayExpire(String.valueOf(userId), date);
	}

	@Override
	public List<Task> getByOrgDeptId(Long topDept, String date) {
		return baseMapper.getByOrgDeptId(topDept, date);
	}

	@Override
	public List<Task> getTodoAndExpireByUserId(Long userId, String date) {
		return baseMapper.getTodoAndExpireByUserId(String.valueOf(userId), date);
	}

	@Override
	public List<Task> getTodayTodoByOrgDeptId(Long topDept, String date) {
		return baseMapper.getTodayTodoByOrgDeptId(topDept, date);
	}

	@Override
	public List<Task> getTodoByYear(Set<Long> projectIds, String year) {
		return baseMapper.getTodoByYear(projectIds, Integer.valueOf(year));
	}

	@Override
	public List<Map<String, String>> projectTaskSummary(Long projectId) {
		List<Map<String, String>> res = new ArrayList<>();
		//项目下任务执行者的情况
		Set<Long> userIds = new HashSet<>();
		List<Task> tasks = this.listByProjectId(projectId);
		if (!CollectionUtils.isEmpty(tasks)) {
			for (Task task : tasks) {
				String executeUser = task.getExecuteUser();
				if (StringUtil.isNotBlank(executeUser)) {
					List<Long> ids = Func.toLongList(executeUser);
					userIds.addAll(ids);
				}
			}
		}
		if (!CollectionUtils.isEmpty(userIds)) {
			userIds.forEach(userId -> {
				Map<String, String> map = new HashMap<>();
				R<User> userR = userClient.userInfoById(userId);
				if (userR.isSuccess()) {
					List<Task> tasks1 = this.listByUserAndProject(userId, projectId);
					if (!CollectionUtils.isEmpty(tasks1)) {
						map.put("value", String.valueOf(tasks1.size()));
						map.put("name", userR.getData().getName());
						//map.put(userR.getData().getName(), tasks1.size());
						res.add(map);
					}
				}
			});
		}
		return res;
	}

	@Override
	public List<Task> listByUserAndProject(Long userId, Long projectId) {
		LambdaQueryWrapper<Task> lqw = new LambdaQueryWrapper<>();
		lqw.eq(Task::getProjectId, projectId);
		lqw.like(Task::getExecuteUser, String.valueOf(userId));
		lqw.orderByAsc(Task::getEndTime);
		return this.list(lqw);
	}
}
