package com.wtkj.wrapper;

import com.wtkj.entity.FileAndFolder;
import com.wtkj.service.IFileAndFolderService;
import com.wtkj.service.IFileReadService;
import com.wtkj.vo.FileAndFolderVO;
import com.wutong.file.feign.IFileClient;
import com.wutong.file.vo.FileVO;
import org.springblade.core.mp.support.BaseEntityWrapper;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.BeanUtil;
import org.springblade.core.tool.utils.SpringUtil;
import org.springblade.system.entity.Dept;
import org.springblade.system.feign.ISysClient;
import org.springblade.system.user.entity.User;
import org.springblade.system.user.feign.IUserClient;
import org.springframework.util.CollectionUtils;

import java.util.List;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;

import static com.wtkj.config.MagicValue.ONE;
import static com.wtkj.config.MagicValue.TWO;

/**
 * @author Blizzard
 * @create at 2023-09-14 17:17
 * @describe
 */
public class FileAndFolderWrapper extends BaseEntityWrapper<FileAndFolder, FileAndFolderVO> {

	private static IUserClient userClient;
	private static ISysClient sysClient;
	private static IFileAndFolderService fileAndFolderService;
	private static IFileClient fileClient;

	private static IFileReadService fileReadService;

	public static FileAndFolderWrapper build() {
		userClient = SpringUtil.getBean(IUserClient.class);
		sysClient = SpringUtil.getBean(ISysClient.class);
		fileAndFolderService = SpringUtil.getBean(IFileAndFolderService.class);
		fileClient = SpringUtil.getBean(IFileClient.class);
		fileReadService = SpringUtil.getBean(IFileReadService.class);
		return new FileAndFolderWrapper();
	}

	@Override
	public FileAndFolderVO entityVO(FileAndFolder entity) {
		FileAndFolderVO vo = Objects.requireNonNull(BeanUtil.copy(entity, FileAndFolderVO.class));
		//创建人
		Long createUser = vo.getCreateUser();
		R<User> userR = userClient.userInfoById(createUser);
		if (userR.isSuccess()) {
			vo.setUploadUser(userR.getData().getName());
		}
		//创建机构
		Long createDept = vo.getCreateDept();
		R<Dept> deptR = sysClient.getDept(createDept);
		if (deptR.isSuccess()) {
			vo.setUploadDept(deptR.getData().getDeptName());
		}
		//如果是文件夹  展示文件夹下文件数量 如果是文件 展示大小和url
		if (vo.getType() != null && vo.getType().equals(TWO)) {
			Integer fileAmount = fileAndFolderService.getFileAmountByFolderId(null, vo.getId());
			vo.setFileAmount(fileAmount);
		} else if (vo.getType() != null && vo.getType().equals(ONE)) {
			R<FileVO> rpc = fileClient.findById(vo.getBladeFileId());
			if (rpc.isSuccess()) {
				vo.setVolume(rpc.getData().getVolume());
				vo.setUrl(rpc.getData().getFilePath());
				vo.setSuffix(rpc.getData().getSuffix());
			}
			//文件阅读人
			List<Long> userIds = fileReadService.getReadUser(vo.getId());
			if (!CollectionUtils.isEmpty(userIds)) {
				Set<User> users = userIds.stream().map(m -> userClient.userInfoById(m)).filter(userR1 -> userR1.isSuccess()).map(R::getData).collect(Collectors.toSet());
				vo.setReadUsers(users);
			}
		}
		//权限
		Integer isPrivate = vo.getIsPrivate();
		Long privateUser = vo.getPrivateUser();
		if (Objects.equals(isPrivate, ONE)) {
			Long userId = AuthUtil.getUserId();
			if (userId.equals(privateUser)) {
				vo.setAuth(ONE);
			}
		}
		return vo;
	}
}
