package com.wtkj.wrapper;

import com.wtkj.entity.TaskLog;
import com.wtkj.service.ITaskLogService;
import com.wtkj.vo.TaskLogVO;
import org.springblade.core.mp.support.BaseEntityWrapper;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.BeanUtil;
import org.springblade.core.tool.utils.SpringUtil;
import org.springblade.system.user.entity.User;
import org.springblade.system.user.feign.IUserClient;
import org.springframework.util.CollectionUtils;

import java.util.List;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @author Blizzard
 * @create at 2023-09-15 16:18
 * @describe
 */
public class TaskLogWrapper extends BaseEntityWrapper<TaskLog, TaskLogVO> {

	private static IUserClient userClient;
	private static ITaskLogService taskLogService;


	public static TaskLogWrapper build() {
		userClient = SpringUtil.getBean(IUserClient.class);
		taskLogService = SpringUtil.getBean(ITaskLogService.class);
		return new TaskLogWrapper();
	}

	@Override
	public TaskLogVO entityVO(TaskLog entity) {
		TaskLogVO vo = Objects.requireNonNull(BeanUtil.copy(entity, TaskLogVO.class));

		//项目组成员
		Long taskId = entity.getTaskId();
		if (taskId != null) {
			List<TaskLog> logs = taskLogService.listByTaskId(taskId);
			if (!CollectionUtils.isEmpty(logs)) {
				Set<User> users = logs.stream().map(TaskLog::getCreateUser).filter(Objects::nonNull).map(m -> userClient.userInfoById(m))
					.filter(f -> f.isSuccess()).map(R::getData).collect(Collectors.toSet());
				vo.setUsers(users);
			}
		}
		return vo;
	}
}
